import { Injectable } from "@angular/core";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { Observable } from "rxjs";
import { User, Login, UserNoPW } from "../models/User";
import { JwtHelperService } from "@auth0/angular-jwt";
import { Board } from "../models/Board";
import * as forge from "node-forge";
let pki = forge.pki;
// 사용자 등록시, 로그인시 헤더 옵.션
const httpOptions = {
  headers: new HttpHeaders({
    "Content-Type": "application/json"
  })
};

@Injectable({
  providedIn: "root"
})
export class AuthService {
  //authToken: any; // 로그인 성공시 생성되는 토큰 정보
  ptoken: any; // 공개토큰
  stoken: any; // 비밀토큰
  userNoPW: UserNoPW; // 로그인 성공시 생성되는 사용자 정보

  constructor(private http: HttpClient, public jwtHelper: JwtHelperService) {}

  prepEndpoint(ep) {
    // 1. localhost에 포팅시
    return "http://localhost:3000/" + ep;

    // 2. Heroku 클라우드 서버에 포팅시
    //return ep;

    // 3. isweb 서버에 포팅시
    //return "http://isweb.joongbu.ac.kr:3000/" + ep;
  }

  registerUser(user): Observable<any> {
    const registerUrl = this.prepEndpoint("users/register");
    return this.http.post<User>(registerUrl, user, httpOptions);
  }
  authenticateUser(login): Observable<any> {
    const loginUrl = this.prepEndpoint("users/authenticate");
    return this.http.post<Login>(loginUrl, login, httpOptions);
  }

  getProfile(ptoken, ctime, auth): Observable<any> {
    this.ptoken = localStorage.getItem("ptoken");

    // 토큰을 포함한 헤더 옵션 생성
    const httpOptions1 = {
      headers: new HttpHeaders({
        "Content-Type": "application/json",
        Authorization: this.ptoken,
        Ctime: ctime,
        Auth: auth
      })
    };
    console.log(httpOptions1);
    const profileUrl = this.prepEndpoint("users/profile");
    return this.http.get(profileUrl, httpOptions1);
  }
  computeAuth(ctime, stoken) {
    let md = forge.md.sha256.create();
    md.update(ctime + stoken);
    return md.digest().toHex();
  }

  getList(): Observable<any> {
    const listUrl = this.prepEndpoint("users/list");
    return this.http.get(listUrl, httpOptions);
  }

  AddtoBoard(adbdata): Observable<any> {
    const registerUrl = this.prepEndpoint("users/list/test1");
    return this.http.post<Board>(registerUrl, adbdata, httpOptions);
  }

  getBoard(): Observable<any> {
    const listUrl = this.prepEndpoint("users/list/test2");
    return this.http.get(listUrl, httpOptions);
  }

  deleteBoard(deleteboard): Observable<any> {
    const deleteUrl = this.prepEndpoint("users/list/test3");
    return this.http.post<Board>(deleteUrl, deleteboard, httpOptions);
  }

  storeUserData(ptoken, stoken, userNoPW) {
    this.ptoken = ptoken;
    this.stoken = stoken;
    this.userNoPW = userNoPW;
    localStorage.setItem("ptoken", ptoken);
    localStorage.setItem("stoken", stoken);
    localStorage.setItem("user", JSON.stringify(userNoPW));
  }

  logout() {
    this.ptoken = null;
    this.stoken = null;
    this.userNoPW = null;
    localStorage.removeItem("ptoken");
    localStorage.removeItem("stoken");
    localStorage.removeItem("user");
  }

  loggedIn() {
    return !this.jwtHelper.isTokenExpired(this.ptoken);
  }
  certRequest(request): Observable<any> {
    // 키쌍 생성
    let keyPair = pki.rsa.generateKeyPair(2048);
    let publicKey = keyPair.publicKey;
    let privateKey = keyPair.privateKey;
    let publicKeyPem = pki.publicKeyToPem(publicKey);
    let privateKeyPem = pki.privateKeyToPem(privateKey); // 개인키 저장
    localStorage.setItem("privateKey", privateKeyPem); // 인증서 발급 요청 req 생성
    const req = {
      country: request.country,
      state: request.state,
      locality: request.locality,
      organization: request.organization,
      orgUnit: request.orgUnit,
      common: request.common,
      publicKey: publicKeyPem
    };
    const certUrl = this.prepEndpoint("users/cert");
    return this.http.post(certUrl, req, httpOptions);
  }
  storeCert(cert, caCert) {
    localStorage.setItem("cert", cert);

    localStorage.setItem("caCert", caCert);
  }
  authenticateSigUser(): Observable<any> {
    const privateKeyPem = localStorage.getItem("privateKey");
    const privateKey = pki.privateKeyFromPem(privateKeyPem);
    const certPem = localStorage.getItem("cert");
    const currentTime = new Date().getTime();
    const cert = pki.certificateFromPem(certPem);
    const cacert = pki.certificateFromPem();

    console.log(cert, "/////////cert/////////");
    const username = cert.subject.getField("CN").value;
    let md = forge.md.sha1.create();
    md.update(username, "utf8");
    md.update(currentTime, "utf8");
    const signature = privateKey.sign(md);
    const signatureHex = forge.util.bytesToHex(signature);
    const request = {
      username: username,
      currentTime: currentTime,
      signatureHex: signatureHex,
      certPem: certPem
    };
    //console.log(request);
    const loginUrl = this.prepEndpoint("users/authenticateSig");
    return this.http.post(loginUrl, request, httpOptions);
  }
}
