const express = require("express");
const router = express.Router();
const passport = require("passport");
const jwt = require("jsonwebtoken");
const Board = require("../models/board");
const User = require("../models/user");
const config = require("../config/database");
const forge = require("node-forge");
const fs = require("fs");
const pki = forge.pki;
const caCertPem = fs.readFileSync("caCert.pem", "utf8");
const caPrivateKeyPem = fs.readFileSync("caPrivateKey.pem", "utf8");
const caCert = pki.certificateFromPem(caCertPem);
const caPrivateKey = pki.privateKeyFromPem(caPrivateKeyPem);
var MongoClient = require("mongodb").MongoClient;
var url =
  "mongodb+srv://hyunsung:tkghl28163@cluster0-stzlk.mongodb.net/test?retryWrites=true&w=majority";

router.post("/list/test1", (req, res, next) => {
  let board = new Board({
    content: req.body.content,
    date: req.body.date,
    number: req.body.number,
    view: req.body.view
  });

  // 시간
  let wtime = new Date();
  board.date = wtime.getFullYear() + "-" + (wtime.getMonth()+1) + "-" + wtime.getDate();

  // 번호
  Board.addBoard(board, (err, data) => {
    if (err) {
      res.json({ success: false, msg: "게시판 등록 실패" });
    } else {
      res.json({ success: true, msg: "게시판 등록 성공" });
    }
  });
});

// Register 사용자등록
router.post("/register", (req, res, next) => {
  let newUser = new User({
    name: req.body.name,
    email: req.body.email,
    username: req.body.username,
    password: req.body.password,
    phone: req.body.phone
  });

  User.getUserByUsername(newUser.username, (err, user) => {
    if (err) throw err;
    if (user) {
      return res.json({
        success: false,
        msg: "동일한 아이디가 존재합니다. 사용자 등록 실패."
      });
    } else {
      User.addUser(newUser, (err, user) => {
        if (err) {
          res.json({ success: false, msg: "사용자 등록 실패" });
        } else {
          res.json({ success: true, msg: "사용자 등록 성공" });
        }
      });
    }
  });
});
router.get("/list/test2", (req, res, next) => {
  Board.getAll((err, boards) => {
    if (err) throw err;
    res.json(boards);
  });
});
router.post("/list/test3", (req, res, next) => {
  let deleteBoardList = new Board({
    content: req.body.content,
    date: req.body.date,
    number: req.body.number
  });

  MongoClient.connect(url, { useNewUrlParser: true }, function (err, db) {
    if (err) throw err;
    var deleteData = db.db("test");
    var numberquery = { number: deleteBoardList.number };

    deleteData
      .collection("boards")
      .findOneAndDelete(numberquery, function (err, text_data) {
        if (err) throw err;
        console.log("complete");
        db.close();
      });
  });
  Board.deleteBoard(deleteBoardList, (err, data) => {
    if (err) {
      res.json({ success: false, msg: "게시판 등록 실패" });
    } else {
      res.json({ success: true, msg: "게시판 등록 성공" });
    }
  });
});

// Authenticate 사용자인증, 로그인
router.post("/authenticate", (req, res, next) => {
  const username = req.body.username;
  const password = req.body.password;

  User.getUserByUsername(username, (err, user) => {
    if (err) throw err;
    if (!user) {
      return res.json({
        success: false,
        msg: "User not found! 등록된 사용자가 없습니다..."
      });
    }
    User.comparePassword(password, user.password, (err, isMatch) => {
      if (err) throw err;
      if (isMatch) {
        const ptoken = jwt.sign({ data: user }, config.secret, {
          expiresIn: 604800 // 1 week, 유효기간: 1주일
        });
        const stoken = jwt.sign({ data: ptoken }, config.secret, {
          noTimestamp: true // 발급시점, 유효기간 삭제.
          // 토큰의 유효성은 ptoken에서만 검증 예정
        });
        res.json({
          success: true,
          ptoken: "JWT " + ptoken,
          stoken: "JWT " + stoken,
          userNoPW: {
            name: user.name,
            username: user.username,
            email: user.email
          }
        });
      } else {
        return res.json({
          success: false,
          msg: "Wrong password. 패스워드가 틀립니다... "
        });
      }
    });
  });
});
// AuthenticateSig 전자서명 간편 로그인
router.post("/authenticateSig", (req, res, next) => {
  const username = req.body.username;
  User.getUserByUsername(username, (err, user) => {
    if (err) throw err;
    if (!user) {
      return res.json({
        success: false,
        msg: "User not found! 등록된 사용자가 없습니다..."
      });
    }
    const currentTime = req.body.currentTime;
    const signatureHex = req.body.signatureHex;
    const certPem = req.body.certPem;
    const cert = pki.certificateFromPem(certPem);
    const publicKey = cert.publicKey;
    const signature = forge.util.hexToBytes(signatureHex);
    const common = cert.subject.getField("CN").value;
    const currentTime1 = new Date().getTime();
    const diffTime = currentTime1 - currentTime;
    let md = forge.md.sha1.create();
    md.update(username, "utf8");
    md.update(currentTime, "utf8");
    let verified1 = publicKey.verify(md.digest().bytes(), signature);
    let verified2 = caCert.verify(cert);
    let verified3;
    if (diffTime < 1000000) verified3 = true;
    let verified4;
    if (username == common) verified4 = true;
    if (
      verified1 == true &&
      verified2 == true &&
      verified3 == true &&
      verified4 == true
    ) {
      const ptoken =
        "JWT " +
        jwt.sign({ data: user }, config.secret, {
          expiresIn: 604800 // 1 week, 유효기간: 1주일
        });
      const stoken =
        "JWT " +
        jwt.sign({ data: ptoken }, config.secret, {
          noTimestamp: true // 발급시점, 유효기간 삭제. 토큰의 유효성은 ptoken에서만 검증 예정
        });
      res.json({
        success: true,
        ptoken: ptoken,
        stoken: stoken,
        userNoPW: {
          name: user.name,
          username: user.username,
          email: user.email,
          age: user.age
        }
      });
    } else {
      return res.json({
        success: false,
        msg: "Login not successful. Something wrong... "
      });
    }
  });
});

// profile 접근은 로그인 상태에서만 토큰을 이용하여 접근하도록 설정
router.get(
  "/profile",
  passport.authenticate("jwt", { session: false }),
  (req, res, next) => {
    const ptoken = req.headers.authorization; // 클라이언트의 공개토큰
    const ctime = req.headers.ctime; // 클라이언트의 현재 시간
    const auth1 = req.headers.auth; // 클라이언트가 보내준 일회용 인증값
    const stoken =
      "JWT " +
      jwt.sign({ data: ptoken }, config.secret, {
        noTimestamp: true // 발급시점, 유효기간 삭제.
      });
    let md = forge.md.sha256.create();
    md.update(ctime + stoken);
    const auth2 = md.digest().toHex(); // 서버가 다시 생성한 일회용 인증값
    const stime = new Date().getTime(); // 서버의 현재 시간
    const diff = stime - ctime.valueOf(); // 서버, 클라이언트의 시간 차이. 정수화하여 뺄셈. 재전송공격 방지.
    if (auth1 == auth2 && diff < 10000) {
      res.json({
        user: {
          name: req.user.name,
          username: req.user.username,
          email: req.user.email
        }
      });
    }
  }
);

router.get("/list", (req, res, next) => {
  User.getAll((err, users) => {
    if (err) throw err;
    res.json(users);
  });
});

router.post("/cert", (req, res, next) => {
  let cert = pki.createCertificate();
  cert.publicKey = pki.publicKeyFromPem(req.body.publicKey);
  cert.serialNumber = "01";
  cert.validity.notBefore = new Date();
  cert.validity.notAfter.setFullYear(cert.validity.notBefore.getFullYear() + 1);
  var userAttrs = [
    {
      shortName: "CN",
      value: req.body.common
    },
    {
      shortName: "C",
      value: req.body.country
    },
    {
      shortName: "ST",
      value: req.body.state
    },
    {
      shortName: "L",
      value: req.body.locality
    },
    {
      shortName: "O",
      value: req.body.organization
    },
    {
      shortName: "OU",
      value: req.body.orgUnit
    }
  ];
  cert.setSubject(userAttrs);
  var caAttrs = [
    {
      shortName: "CN",
      value: caCert.subject.getField("CN").value
    },
    {
      shortName: "C",
      value: caCert.subject.getField("C").value
    },
    {
      shortName: "ST",
      value: caCert.subject.getField("ST").value
    },
    {
      shortName: "L",
      value: caCert.subject.getField("L").value
    },
    {
      shortName: "O",
      value: caCert.subject.getField("O").value
    },
    {
      shortName: "OU",
      value: caCert.subject.getField("OU").value
    }
  ];
  cert.setIssuer(caAttrs);
  cert.setExtensions([
    {
      name: "basicConstraints",
      cA: true
    },
    {
      name: "keyUsage",
      keyCertSign: true,
      digitalSignature: true,
      nonRepudiation: true,
      keyEncipherment: true,
      dataEncipherment: true
    },
    {
      name: "extKeyUsage",
      serverAuth: true,
      clientAuth: true,
      codeSigning: true,
      emailProtection: true,
      timeStamping: true
    },
    {
      name: "nsCertType",
      client: true,
      server: true,
      email: true,
      objsign: true,
      sslCA: true,
      emailCA: true,
      objCA: true
    },
    {
      name: "subjectAltName",
      altNames: [
        {
          type: 6, // URI
          value: "http://example.org/"
        },
        {
          type: 7, // IP
          ip: "127.0.0.1"
        }
      ]
    },
    {
      name: "subjectKeyIdentifier"
    }
  ]);
  cert.sign(caPrivateKey);
  return res.json({
    success: true,
    cert: pki.certificateToPem(cert),
    caCert: caCertPem
  });
});

module.exports = router;
